@@ -19,7 +19,7 @@ table.create()
 
 mapper(Tree, table,
               properties={
-                'childs':relation(Tree, foreignkey=table.c.father_id, primaryjoin=table.c.father_id==table.c.id,  backref=backref('father', uselist=False, foreignkey=table.c.id))},
+                'childs':relation(Tree, remote_side=table.c.father_id, primaryjoin=table.c.father_id==table.c.id,  backref=backref('father', remote_side=table.c.id))},
             )
 
 root = Tree('root')
@@ -27,6 +27,9 @@ child1 = Tree('child1', root)
 child2 = Tree('child2', root)
 child3 = Tree('child3', child1)
 
+child4 = Tree('child4')
+child1.childs.append(child4)
+
 session = create_session()
 session.save(root)
 session.flush()
