@@ -1,36 +1,45 @@
 # mssql.py
 
-"""
-notes:
-  supports the pymssq, adodbapi and pyodbc interfaces
+"""MSSQL backend, thru either pymssq, adodbapi or pyodbc interfaces.
 
-  IDENTITY columns are supported by using SA schema.Sequence() objects. In other words:
-         Table('test', mss_engine,
-                Column('id',   Integer, Sequence('blah',100,10), primary_key=True),
-                Column('name', String(20))
-              ).create()
+* ``IDENTITY`` columns are supported by using SA ``schema.Sequence()``
+  objects. In other words::
 
-         would yield:
-         CREATE TABLE test (
-           id INTEGER NOT NULL IDENTITY(100,10) PRIMARY KEY,
-           name VARCHAR(20)
-           )
-  note that the start & increment values for sequences are optional and will default to 1,1
+    Table('test', mss_engine,
+           Column('id',   Integer, Sequence('blah',100,10), primary_key=True),
+           Column('name', String(20))
+         ).create()
 
-  support for SET IDENTITY_INSERT ON mode (automagic on / off for INSERTs)
+  would yield::
 
-  support for auto-fetching of @@IDENTITY on insert
+   CREATE TABLE test (
+     id INTEGER NOT NULL IDENTITY(100,10) PRIMARY KEY,
+     name VARCHAR(20)
+     )
 
-  select.limit implemented as SELECT TOP n
+  Note that the start & increment values for sequences are optional
+  and will default to 1,1.
+
+* Support for ``SET IDENTITY_INSERT ON`` mode (automagic on / off for 
+  ``INSERT`` s)
+
+* Support for auto-fetching of ``@@IDENTITY`` on ``INSERT``
+
+* ``select.limit`` implemented as ``SELECT TOP n``
 
 
 Known issues / TODO:
-  no support for more than one IDENTITY column per table
-  no support for table reflection of IDENTITY columns with (seed,increment) values other than (1,1)
-  no support for GUID type columns (yet)
-  pymssql has problems with binary and unicode data that this module does NOT work around
-  adodbapi fails testtypes.py unit test on unicode data too -- issue with the test?
 
+* No support for more than one ``IDENTITY`` column per table
+
+* No support for table reflection of ``IDENTITY`` columns with
+  (seed,increment) values other than (1,1)
+
+* No support for ``GUID`` type columns (yet)
+
+* pymssql has problems with binary and unicode data that this module
+  does **not** work around
+  
 """
 
 import sys, StringIO, string, types, re, datetime
@@ -43,80 +52,6 @@ import sqlalchemy.ansisql as ansisql
 import sqlalchemy.types as sqltypes
 import sqlalchemy.exceptions as exceptions
 
-dbmodule = None
-dialect = None
-
-def use_adodbapi():
-    global dbmodule, connect, make_connect_string, do_commit, sane_rowcount, dialect, colspecs, ischema_names
-    import adodbapi as dbmodule
-    # ADODBAPI has a non-standard Connection method
-    connect = dbmodule.Connection
-    def make_connect_string(keys):
-        return  [["Provider=SQLOLEDB;Data Source=%s;User Id=%s;Password=%s;Initial Catalog=%s" % (
-            keys.get("host"), keys.get("user"), keys.get("password", ""), keys.get("database"))], {}]        
-    sane_rowcount = True
-    dialect = MSSQLDialect
-    colspecs[sqltypes.Unicode] = AdoMSUnicode
-    ischema_names['nvarchar'] = AdoMSUnicode
-    
-def use_pymssql():
-    global dbmodule, connect, make_connect_string, do_commit, sane_rowcount, dialect, colspecs, ischema_names
-    import pymssql as dbmodule
-    connect = dbmodule.connect
-    # pymmsql doesn't have a Binary method.  we use string
-    dbmodule.Binary = lambda st: str(st)
-    def make_connect_string(keys):
-        if keys.get('port'):
-            # pymssql expects port as host:port, not a separate arg
-            keys['host'] = ''.join([keys.get('host', ''), ':', str(keys['port'])])
-            del keys['port'] 
-        return [[], keys]
-    do_commit = True
-    sane_rowcount = False
-    dialect = PyMSSQLDialect
-    colspecs[sqltypes.Unicode] = MSUnicode
-    ischema_names['nvarchar'] = MSUnicode
-    
-def use_pyodbc():
-    global dbmodule, connect, make_connect_string, do_commit, sane_rowcount, dialect, colspecs, ischema_names
-    import pyodbc as dbmodule
-    connect = dbmodule.connect
-    def make_connect_string(keys):
-        return [["Driver={SQL Server};Server=%s;UID=%s;PWD=%s;Database=%s" % (
-            keys.get("host"), keys.get("user"), keys.get("password", ""), keys.get("database"))], {}]        
-    do_commit = True
-    sane_rowcount = False
-    dialect = MSSQLDialect
-    import warnings
-    warnings.warn('pyodbc support in sqlalchemy.databases.mssql is experimental - use at your own risk.')
-    colspecs[sqltypes.Unicode] = AdoMSUnicode
-    ischema_names['nvarchar'] = AdoMSUnicode
-
-def use_default():
-    import_errors = []
-    def try_use(f):
-        try:
-            f()
-        except ImportError, e:
-            import_errors.append(e)
-            return False
-        else:
-            return True
-    for f in [
-            # XXX - is this the best default ordering? For now, it retains the current (2007-Jan-11) 
-            # default - that is, adodbapi first, pymssql second - and adds pyodbc as a third option.
-            # However, my tests suggest that the exact opposite order may be the best!
-            use_adodbapi,
-            use_pymssql,
-            use_pyodbc,
-            ]:
-        if try_use(f):
-            return dbmodule # informational return, so the user knows what he's using.
-    else:
-        return None
-        # cant raise this right now since all dialects need to be importable/loadable
-        #raise ImportError(import_errors)
-        
 
 class MSNumeric(sqltypes.Numeric):
     def convert_result_value(self, value, dialect):
@@ -138,9 +73,10 @@ class MSNumeric(sqltypes.Numeric):
 class MSFloat(sqltypes.Float):
     def get_col_spec(self):
         return "FLOAT(%(precision)s)" % {'precision': self.precision}
+
     def convert_bind_param(self, value, dialect):
         """By converting to string, we can use Decimal types round-trip."""
-        return str(value) 
+        return value and str(value) or None
 
 class MSInteger(sqltypes.Integer):
     def get_col_spec(self):
@@ -185,7 +121,7 @@ class MSDate(sqltypes.Date):
     
     def convert_bind_param(self, value, dialect):
         if value and hasattr(value, "isoformat"):
-            return value.strftime('%Y-%m-%d %H:%M:%S')
+            return value.strftime('%Y-%m-%d %H:%M')
         return value
 
     def convert_result_value(self, value, dialect):
@@ -196,54 +132,68 @@ class MSDate(sqltypes.Date):
 
 class MSText(sqltypes.TEXT):
     def get_col_spec(self):
-        return "TEXT"
+        if self.dialect.text_as_varchar:
+            return "VARCHAR(max)"            
+        else:
+            return "TEXT"
+
 class MSString(sqltypes.String):
     def get_col_spec(self):
         return "VARCHAR(%(length)s)" % {'length' : self.length}
 
-
 class MSNVarchar(MSString):
-    """NVARCHAR string, does unicode conversion if dialect.convert_encoding is true"""
+    """NVARCHAR string, does Unicode conversion if `dialect.convert_encoding` is True.  """
     impl = sqltypes.Unicode
+
     def get_col_spec(self):
         if self.length:
             return "NVARCHAR(%(length)s)" % {'length' : self.length}
+        elif self.dialect.text_as_varchar:
+            return "NVARCHAR(max)"
         else:
             return "NTEXT"
 
 class AdoMSNVarchar(MSNVarchar):
     def convert_bind_param(self, value, dialect):
         return value
+
     def convert_result_value(self, value, dialect):
         return value        
 
 class MSUnicode(sqltypes.Unicode):
-    """Unicode subclass, does unicode conversion in all cases, uses NVARCHAR impl"""
+    """Unicode subclass, does Unicode conversion in all cases, uses NVARCHAR impl."""
     impl = MSNVarchar
 
 class AdoMSUnicode(MSUnicode):
     impl = AdoMSNVarchar
+
     def convert_bind_param(self, value, dialect):
         return value
+
     def convert_result_value(self, value, dialect):
         return value        
 
 class MSChar(sqltypes.CHAR):
     def get_col_spec(self):
         return "CHAR(%(length)s)" % {'length' : self.length}
+
 class MSNChar(sqltypes.NCHAR):
     def get_col_spec(self):
         return "NCHAR(%(length)s)" % {'length' : self.length}
+
 class MSBinary(sqltypes.Binary):
     def get_col_spec(self):
         return "IMAGE"
+
 class MSBoolean(sqltypes.Boolean):
     def get_col_spec(self):
         return "BIT"
+
     def convert_result_value(self, value, dialect):
         if value is None:
             return None
         return value and True or False
+
     def convert_bind_param(self, value, dialect):
         if value is True:
             return 1
@@ -254,43 +204,6 @@ class MSBoolean(sqltypes.Boolean):
         else:
             return value and True or False
         
-colspecs = {
-    sqltypes.Integer : MSInteger,
-    sqltypes.Smallinteger: MSSmallInteger,
-    sqltypes.Numeric : MSNumeric,
-    sqltypes.Float : MSFloat,
-    sqltypes.DateTime : MSDateTime,
-    sqltypes.Date : MSDate,
-    sqltypes.String : MSString,
-    sqltypes.Unicode : MSUnicode,
-    sqltypes.Binary : MSBinary,
-    sqltypes.Boolean : MSBoolean,
-    sqltypes.TEXT : MSText,
-    sqltypes.CHAR: MSChar,
-    sqltypes.NCHAR: MSNChar,
-}
-
-ischema_names = {
-    'int' : MSInteger,
-    'smallint' : MSSmallInteger,
-    'tinyint' : MSTinyInteger,
-    'varchar' : MSString,
-    'nvarchar' : MSUnicode,
-    'char' : MSChar,
-    'nchar' : MSNChar,
-    'text' : MSText,
-    'ntext' : MSText, 
-    'decimal' : MSNumeric,
-    'numeric' : MSNumeric,
-    'float' : MSFloat,
-    'datetime' : MSDateTime,
-    'smalldatetime' : MSDate,
-    'binary' : MSBinary,
-    'bit': MSBoolean,
-    'real' : MSFloat,
-    'image' : MSBinary
-}
-
 def descriptor():
     return {'name':'mssql',
     'description':'MSSQL',
@@ -305,29 +218,36 @@ class MSSQLExecutionContext(default.DefaultExecutionContext):
     def __init__(self, dialect):
         self.IINSERT = self.HASIDENT = False
         super(MSSQLExecutionContext, self).__init__(dialect)
-    
+
+    def _has_implicit_sequence(self, column):
+        if column.primary_key and column.autoincrement:
+            if isinstance(column.type, sqltypes.Integer) and not column.foreign_key:
+                if column.default is None or (isinstance(column.default, schema.Sequence) and \
+                                              column.default.optional):
+                    return True
+        return False
+
     def pre_exec(self, engine, proxy, compiled, parameters, **kwargs):
-        """ MS-SQL has a special mode for inserting non-NULL values into IDENTITY columns.
-        Activate it if the feature is turned on and needed. """
+        """MS-SQL has a special mode for inserting non-NULL values
+        into IDENTITY columns.
+        
+        Activate it if the feature is turned on and needed.
+        """
         if getattr(compiled, "isinsert", False):
             tbl = compiled.statement.table
-            if not hasattr(tbl, 'has_sequence'):                
+            if not hasattr(tbl, 'has_sequence'):
+                tbl.has_sequence = False
                 for column in tbl.c:
-                    if column.primary_key and column.autoincrement and \
-                           isinstance(column.type, sqltypes.Integer) and not column.foreign_key:
-                        if column.default is None or (isinstance(column.default, schema.Sequence) and \
-                                                      column.default.optional):
-                            tbl.has_sequence = column
-                            break
-                else:
-                    tbl.has_sequence = False
+                    if getattr(column, 'sequence', False) or self._has_implicit_sequence(column):
+                        tbl.has_sequence = column
+                        break
 
             self.HASIDENT = bool(tbl.has_sequence)
             if engine.dialect.auto_identity_insert and self.HASIDENT:
                 if isinstance(parameters, list):
-                    self.IINSERT = parameters[0].has_key(tbl.has_sequence.name)
+                    self.IINSERT = parameters[0].has_key(tbl.has_sequence.key)
                 else:
-                    self.IINSERT = parameters.has_key(tbl.has_sequence.name)
+                    self.IINSERT = parameters.has_key(tbl.has_sequence.key)
             else:
                 self.IINSERT = False
 
@@ -337,7 +257,11 @@ class MSSQLExecutionContext(default.DefaultExecutionContext):
         super(MSSQLExecutionContext, self).pre_exec(engine, proxy, compiled, parameters, **kwargs)
 
     def post_exec(self, engine, proxy, compiled, parameters, **kwargs):
-        """ Turn off the INDENTITY_INSERT mode if it's been activated, and fetch recently inserted IDENTIFY values (works only for one column) """
+        """Turn off the INDENTITY_INSERT mode if it's been activated,
+        and fetch recently inserted IDENTIFY values (works only for
+        one column).
+        """
+        
         if getattr(compiled, "isinsert", False):
             if self.IINSERT:
                 proxy("SET IDENTITY_INSERT %s OFF" % compiled.statement.table.name)
@@ -351,12 +275,72 @@ class MSSQLExecutionContext(default.DefaultExecutionContext):
 
 
 class MSSQLDialect(ansisql.ANSIDialect):
-    def __init__(self, module=None, auto_identity_insert=True, **params):
-        self.module = module or dbmodule or use_default()
+    colspecs = {
+        sqltypes.Integer : MSInteger,
+        sqltypes.Smallinteger: MSSmallInteger,
+        sqltypes.Numeric : MSNumeric,
+        sqltypes.Float : MSFloat,
+        sqltypes.DateTime : MSDateTime,
+        sqltypes.Date : MSDate,
+        sqltypes.String : MSString,
+        sqltypes.Unicode : MSUnicode,
+        sqltypes.Binary : MSBinary,
+        sqltypes.Boolean : MSBoolean,
+        sqltypes.TEXT : MSText,
+        sqltypes.CHAR: MSChar,
+        sqltypes.NCHAR: MSNChar,
+    }
+
+    ischema_names = {
+        'int' : MSInteger,
+        'smallint' : MSSmallInteger,
+        'tinyint' : MSTinyInteger,
+        'varchar' : MSString,
+        'nvarchar' : MSUnicode,
+        'char' : MSChar,
+        'nchar' : MSNChar,
+        'text' : MSText,
+        'ntext' : MSText,
+        'decimal' : MSNumeric,
+        'numeric' : MSNumeric,
+        'float' : MSFloat,
+        'datetime' : MSDateTime,
+        'smalldatetime' : MSDate,
+        'binary' : MSBinary,
+        'bit': MSBoolean,
+        'real' : MSFloat,
+        'image' : MSBinary
+    }
+
+    def __new__(cls, module_name=None, *args, **kwargs):
+        module = kwargs.get('module', None)
+        if cls != MSSQLDialect:
+            return super(MSSQLDialect, cls).__new__(cls, *args, **kwargs)
+        if module_name:
+            dialect = dialect_mapping.get(module_name)
+            if not dialect:
+                raise exceptions.InvalidRequestError('Unsupported MSSQL module requested (must be adodbpi, pymssql or pyodbc): ' + module_name)
+            if not hasattr(dialect, 'module'):
+                raise dialect.saved_import_error
+            return dialect(*args, **kwargs)
+        elif module:
+            return object.__new__(cls, *args, **kwargs)
+        else:
+            for dialect in dialect_preference:
+                if hasattr(dialect, 'module'):
+                    return dialect(*args, **kwargs)
+            #raise ImportError('No DBAPI module detected for MSSQL - please install adodbapi, pymssql or pyodbc')
+            else:
+                return object.__new__(cls, *args, **kwargs)
+                
+    def __init__(self, module_name=None, module=None, auto_identity_insert=True, **params):
+        if not hasattr(self, 'module'):
+            self.module = module
+        super(MSSQLDialect, self).__init__(**params)
         self.auto_identity_insert = auto_identity_insert
-        ansisql.ANSIDialect.__init__(self, **params)
+        self.text_as_varchar = False
         self.set_default_schema_name("dbo")
-        
+            
     def create_connect_args(self, url):
         opts = url.translate_connect_args(['host', 'database', 'user', 'password', 'port'])
         opts.update(url.query)
@@ -364,19 +348,26 @@ class MSSQLDialect(ansisql.ANSIDialect):
             self.auto_identity_insert = bool(opts.pop('auto_identity_insert'))
         if opts.has_key('query_timeout'):
             self.query_timeout = int(opts.pop('query_timeout'))
-        return make_connect_string(opts)
+        if opts.has_key('text_as_varchar'):
+            self.text_as_varchar = bool(opts.pop('text_as_varchar'))
+        return self.make_connect_string(opts)
 
     def create_execution_context(self):
         return MSSQLExecutionContext(self)
 
     def type_descriptor(self, typeobj):
-        return sqltypes.adapt_type(typeobj, colspecs)
+        newobj = sqltypes.adapt_type(typeobj, self.colspecs)
+        # Some types need to know about the dialect
+        if isinstance(newobj, (MSText, MSNVarchar)):
+            newobj.dialect = self
+        return newobj
 
     def last_inserted_ids(self):
         return self.context.last_inserted_ids
 
+    # this is only implemented in the dbapi-specific subclasses
     def supports_sane_rowcount(self):
-        return sane_rowcount
+        raise NotImplementedError()
 
     def compiler(self, statement, bindparams, **kwargs):
         return MSSQLCompiler(self, statement, bindparams, **kwargs)
@@ -429,8 +420,7 @@ class MSSQLDialect(ansisql.ANSIDialect):
         c = self._pool.connect()
         c.supportsTransactions = 0
         return c
-
-          
+  
     def dbapi(self):
         return self.module
 
@@ -495,8 +485,13 @@ class MSSQLDialect(ansisql.ANSIDialect):
             for a in (charlen, numericprec, numericscale):
                 if a is not None:
                     args.append(a)
-            coltype = ischema_names[type]
-            coltype = coltype(*args)
+            coltype = self.ischema_names[type]
+            if coltype == MSString and charlen == -1:
+                coltype = MSText()                
+            else:
+                if coltype == MSNVarchar and charlen == -1:
+                    charlen = None
+                coltype = coltype(*args)
             colargs= []
             if default is not None:
                 colargs.append(schema.PassiveDefault(sql.text(default)))
@@ -535,7 +530,6 @@ class MSSQLDialect(ansisql.ANSIDialect):
             if 'PRIMARY' in row[TC.c.constraint_type.name]:
                 table.primary_key.add(table.c[row[0]])
 
-
         # Foreign key constraints
         s = sql.select([C.c.column_name,
                         R.c.table_schema, R.c.table_name, R.c.column_name,
@@ -546,7 +540,7 @@ class MSSQLDialect(ansisql.ANSIDialect):
                                 R.c.constraint_name == RR.c.unique_constraint_name,
                                 C.c.ordinal_position == R.c.ordinal_position
                                 ),
-                       order_by = [RR.c.constraint_name])
+                       order_by = [RR.c.constraint_name, R.c.ordinal_position])
         rows = connection.execute(s).fetchall()
 
         # group rows by constraint ID, to handle multi-column FKs
@@ -562,10 +556,18 @@ class MSSQLDialect(ansisql.ANSIDialect):
 
         if fknm and scols:
             table.append_constraint(schema.ForeignKeyConstraint(scols, ['%s.%s' % (t,c) for (s,t,c) in rcols], fknm))
-                                
 
+class MSSQLDialect_pymssql(MSSQLDialect):
+    try:
+        import pymssql as module
+        # pymmsql doesn't have a Binary method.  we use string
+        module.Binary = lambda st: str(st)
+    except ImportError, e:
+        saved_import_error = e
+
+    def supports_sane_rowcount(self):
+        return True
 
-class PyMSSQLDialect(MSSQLDialect):
     def do_rollback(self, connection):
         # pymssql throws an error on repeated rollbacks. Ignore it.
         try:
@@ -574,11 +576,18 @@ class PyMSSQLDialect(MSSQLDialect):
             pass
 
     def create_connect_args(self, url):
-        r = super(PyMSSQLDialect, self).create_connect_args(url)
+        r = super(MSSQLDialect_pymssql, self).create_connect_args(url)
         if hasattr(self, 'query_timeout'):
-            dbmodule._mssql.set_query_timeout(self.query_timeout)
+            self.module._mssql.set_query_timeout(self.query_timeout)
         return r
-        
+
+    def make_connect_string(self, keys):
+        if keys.get('port'):
+            # pymssql expects port as host:port, not a separate arg
+            keys['host'] = ''.join([keys.get('host', ''), ':', str(keys['port'])])
+            del keys['port']
+        return [[], keys]
+
 
 ##    This code is leftover from the initial implementation, for reference
 ##    def do_begin(self, connection):
@@ -611,6 +620,67 @@ class PyMSSQLDialect(MSSQLDialect):
 ##        r.query("begin tran")
 ##        r.fetch_array()
 
+class MSSQLDialect_pyodbc(MSSQLDialect):
+    try:
+        import pyodbc as module
+    except ImportError, e:
+        saved_import_error = e
+
+    colspecs = MSSQLDialect.colspecs.copy()
+    colspecs[sqltypes.Unicode] = AdoMSUnicode
+    ischema_names = MSSQLDialect.ischema_names.copy()
+    ischema_names['nvarchar'] = AdoMSUnicode
+
+    def supports_sane_rowcount(self):
+        return False
+
+    def make_connect_string(self, keys):
+        connectors = ["Driver={SQL Server}"]
+        connectors.append("Server=%s" % keys.get("host"))
+        connectors.append("Database=%s" % keys.get("database"))
+        user = keys.get("user")
+        if user:
+            connectors.append("UID=%s" % user)
+            connectors.append("PWD=%s" % keys.get("password", ""))
+        else:
+            connectors.append ("TrustedConnection=Yes")
+        return [[";".join (connectors)], {}]
+
+
+class MSSQLDialect_adodbapi(MSSQLDialect):
+    try:
+        import adodbapi as module
+    except ImportError, e:
+        saved_import_error = e
+
+    colspecs = MSSQLDialect.colspecs.copy()
+    colspecs[sqltypes.Unicode] = AdoMSUnicode
+    ischema_names = MSSQLDialect.ischema_names.copy()
+    ischema_names['nvarchar'] = AdoMSUnicode
+
+    def supports_sane_rowcount(self):
+        return True
+
+    def make_connect_string(self, keys):
+        connectors = ["Provider=SQLOLEDB"]
+        connectors.append ("Data Source=%s" % keys.get("host"))
+        connectors.append ("Initial Catalog=%s" % keys.get("database"))
+        user = keys.get("user")
+        if user:
+            connectors.append("User Id=%s" % user)
+            connectors.append("Password=%s" % keys.get("password", ""))
+        else:
+            connectors.append("Integrated Security=SSPI")
+        return [[";".join (connectors)], {}]
+
+
+dialect_mapping = {
+    'pymssql':  MSSQLDialect_pymssql,
+    'pyodbc':   MSSQLDialect_pyodbc,
+    'adodbapi': MSSQLDialect_adodbapi
+    }
+dialect_preference = [MSSQLDialect_adodbapi, MSSQLDialect_pymssql, MSSQLDialect_pyodbc]
+
 
 class MSSQLCompiler(ansisql.ANSICompiler):
     def __init__(self, dialect, statement, parameters, **kwargs):
@@ -620,25 +690,26 @@ class MSSQLCompiler(ansisql.ANSICompiler):
     def visit_select_precolumns(self, select):
         """ MS-SQL puts TOP, it's version of LIMIT here """
         s = select.distinct and "DISTINCT " or ""
-        if (select.limit):
+        if select.limit:
             s += "TOP %s " % (select.limit,)
+        if select.offset:
+            raise exceptions.InvalidRequestError('MSSQL does not support LIMIT with an offset')
         return s
 
-    def limit_clause(self, select):
-        # Limit in mssql is after the select keyword; MSsql has no support for offset
+    def limit_clause(self, select):    
+        # Limit in mssql is after the select keyword
         return ""
-
             
     def visit_table(self, table):
         # alias schema-qualified tables
         if getattr(table, 'schema', None) is not None and not self.tablealiases.has_key(table):
             alias = table.alias()
             self.tablealiases[table] = alias
-            alias.accept_visitor(self)
+            self.traverse(alias)
             self.froms[('alias', table)] = self.froms[table]
             for c in alias.c:
-                c.accept_visitor(self)
-            alias.oid_column.accept_visitor(self)
+                self.traverse(c)
+            self.traverse(alias.oid_column)
             self.tablealiases[alias] = self.froms[table]
             self.froms[table] = self.froms[alias]
         else:
@@ -676,6 +747,16 @@ class MSSQLCompiler(ansisql.ANSICompiler):
         # "FOR UPDATE" is only allowed on "DECLARE CURSOR" which SQLAlchemy doesn't use
         return ''
 
+    def order_by_clause(self, select):
+        order_by = self.get_str(select.order_by_clause)
+
+        # MSSQL only allows ORDER BY in subqueries if there is a LIMIT
+        if order_by and (not select.is_subquery or select.limit):
+            return " ORDER BY " + order_by
+        else:
+            return ""
+
+
 class MSSQLSchemaGenerator(ansisql.ANSISchemaGenerator):
     def get_column_specification(self, column, **kwargs):
         colspec = self.preparer.format_column(column) + " " + column.type.engine_impl(self.engine).get_col_spec()
@@ -699,7 +780,6 @@ class MSSQLSchemaGenerator(ansisql.ANSISchemaGenerator):
         
         return colspec
 
-
 class MSSQLSchemaDropper(ansisql.ANSISchemaDropper):
     def visit_index(self, index):
         self.append("\nDROP INDEX " + index.table.name + "." + index.name)
@@ -711,12 +791,14 @@ class MSSQLDefaultRunner(ansisql.ANSIDefaultRunner):
 class MSSQLIdentifierPreparer(ansisql.ANSIIdentifierPreparer):
     def __init__(self, dialect):
         super(MSSQLIdentifierPreparer, self).__init__(dialect, initial_quote='[', final_quote=']')
+
     def _escape_identifier(self, value):
         #TODO: determin MSSQL's escapeing rules
         return value
+
     def _fold_identifier_case(self, value):
         #TODO: determin MSSQL's case folding rules
         return value
 
-use_default()
+dialect = MSSQLDialect
 
