@@ -20,6 +20,8 @@ class MockConnection(object):
         mcid += 1
     def close(self):
         pass
+    def rollback(self):
+        pass
     def cursor(self):
         return MockCursor()
 class MockCursor(object):
@@ -33,7 +35,7 @@ class PoolTest(PersistTest):
         pool.clear_managers()
 
     def testmanager(self):
-        manager = pool.manage(mock_dbapi)
+        manager = pool.manage(mock_dbapi, use_threadlocal=True)
         
         connection = manager.connect('foo.db')
         connection2 = manager.connect('foo.db')
@@ -160,7 +162,7 @@ class PoolTest(PersistTest):
         c2 = p.connect()
         assert id(c2.connection) == c_id
         c2.close()
-        time.sleep(3)
+        time.sleep(4)
         c3= p.connect()
         assert id(c3.connection) != c_id
     
@@ -170,7 +172,6 @@ class PoolTest(PersistTest):
         c1 = p.connect()
         c_id = c1.connection.id
         c1.close(); c1=None
-
         c1 = p.connect()
         assert c1.connection.id == c_id
         c1.invalidate()
