@@ -17,4 +17,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.runTests(suite())
+    testbase.main(suite())
