@@ -498,6 +498,282 @@ class RelationTest4(testbase.ORMTest):
         sess.flush()
         assert a1 not in sess
         assert b1 not in sess
+
+class RelationTest5(testbase.ORMTest):
+    """test a map to a select that relates to a map to the table"""
+    def define_tables(self, metadata):
+        global items
+        items = Table('items', metadata,
+            Column('item_policy_num', String(10), primary_key=True, key='policyNum'),
+            Column('item_policy_eff_date', Date, primary_key=True, key='policyEffDate'),
+            Column('item_type', String(20), primary_key=True, key='type'),
+            Column('item_id', Integer, primary_key=True, key='id'),
+        )
+
+    def test_basic(self):
+        class Container(object):pass
+        class LineItem(object):pass
+        
+        container_select = select(
+            [items.c.policyNum, items.c.policyEffDate, items.c.type],
+            distinct=True,
+            ).alias('container_select')
+
+        mapper(LineItem, items)
+
+        mapper(Container, container_select, order_by=asc(container_select.c.type), properties=dict(
+            lineItems = relation(LineItem, lazy=True, cascade='all, delete-orphan', order_by=asc(items.c.type),
+                primaryjoin=and_(
+                    container_select.c.policyNum==items.c.policyNum,
+                    container_select.c.policyEffDate==items.c.policyEffDate,
+                    container_select.c.type==items.c.type
+                ),
+                foreign_keys=[
+                    items.c.policyNum,
+                    items.c.policyEffDate,
+                    items.c.type,
+                ],
+            )
+        ))
+        session = create_session()
+        con = Container()
+        con.policyNum = "99"
+        con.policyEffDate = datetime.date.today()
+        con.type = "TESTER"
+        session.save(con)
+        for i in range(0, 10):
+            li = LineItem()
+            li.id = i
+            con.lineItems.append(li)
+            session.save(li)
+        session.flush()
+        session.clear()
+        newcon = session.query(Container).selectfirst()
+        assert con.policyNum == newcon.policyNum
+        assert len(newcon.lineItems) == 10
+        for old, new in zip(con.lineItems, newcon.lineItems):
+            assert old.id == new.id
+        
+        
+class TypeMatchTest(testbase.ORMTest):
+    """test errors raised when trying to add items whose type is not handled by a relation"""
+    def define_tables(self, metadata):
+        global a, b, c, d
+        a = Table("a", metadata, 
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)))
+        b = Table("b", metadata, 
+            Column('id', Integer, primary_key=True),
+            Column("a_id", Integer, ForeignKey("a.id")),
+            Column('data', String(30)))
+        c = Table("c", metadata, 
+            Column('id', Integer, primary_key=True),
+            Column("b_id", Integer, ForeignKey("b.id")),
+            Column('data', String(30)))
+        d = Table("d", metadata, 
+            Column('id', Integer, primary_key=True),
+            Column("a_id", Integer, ForeignKey("a.id")),
+            Column('data', String(30)))
+    def test_o2m_oncascade(self):
+        class A(object):pass
+        class B(object):pass
+        class C(object):pass
+        mapper(A, a, properties={'bs':relation(B)})
+        mapper(B, b)
+        mapper(C, c)
+        
+        a1 = A()
+        b1 = B()
+        c1 = C()
+        a1.bs.append(b1)
+        a1.bs.append(c1)
+        sess = create_session()
+        try:
+            sess.save(a1)
+            assert False
+        except exceptions.AssertionError, err:
+            assert str(err) == "Attribute 'bs' on class '%s' doesn't handle objects of type '%s'" % (A, C)
+    def test_o2m_onflush(self):
+        class A(object):pass
+        class B(object):pass
+        class C(object):pass
+        mapper(A, a, properties={'bs':relation(B, cascade="none")})
+        mapper(B, b)
+        mapper(C, c)
+        
+        a1 = A()
+        b1 = B()
+        c1 = C()
+        a1.bs.append(b1)
+        a1.bs.append(c1)
+        sess = create_session()
+        sess.save(a1)
+        sess.save(b1)
+        sess.save(c1)
+        try:
+            sess.flush()
+            assert False
+        except exceptions.FlushError, err:
+            assert str(err).startswith("Attempting to flush an item of type %s on collection 'A.bs (B)', which is handled by mapper 'Mapper|B|b' and does not load items of that type.  Did you mean to use a polymorphic mapper for this relationship ?" % C)
+    def test_o2m_nopoly_onflush(self):
+        class A(object):pass
+        class B(object):pass
+        class C(B):pass
+        mapper(A, a, properties={'bs':relation(B, cascade="none")})
+        mapper(B, b)
+        mapper(C, c, inherits=B)
+        
+        a1 = A()
+        b1 = B()
+        c1 = C()
+        a1.bs.append(b1)
+        a1.bs.append(c1)
+        sess = create_session()
+        sess.save(a1)
+        sess.save(b1)
+        sess.save(c1)
+        try:
+            sess.flush()
+            assert False
+        except exceptions.FlushError, err:
+            assert str(err).startswith("Attempting to flush an item of type %s on collection 'A.bs (B)', which is handled by mapper 'Mapper|B|b' and does not load items of that type.  Did you mean to use a polymorphic mapper for this relationship ?" % C)
     
+    def test_m2o_nopoly_onflush(self):
+        class A(object):pass
+        class B(A):pass
+        class D(object):pass
+        mapper(A, a)
+        mapper(B, b, inherits=A)
+        mapper(D, d, properties={"a":relation(A, cascade="none")})
+        b1 = B()
+        d1 = D()
+        d1.a = b1
+        sess = create_session()
+        sess.save(b1)
+        sess.save(d1)
+        try:
+            sess.flush()
+            assert False
+        except exceptions.FlushError, err:
+            assert str(err).startswith("Attempting to flush an item of type %s on collection 'D.a (A)', which is handled by mapper 'Mapper|A|a' and does not load items of that type.  Did you mean to use a polymorphic mapper for this relationship ?" % B)
+    def test_m2o_oncascade(self):
+        class A(object):pass
+        class B(object):pass
+        class D(object):pass
+        mapper(A, a)
+        mapper(B, b)
+        mapper(D, d, properties={"a":relation(A)})
+        b1 = B()
+        d1 = D()
+        d1.a = b1
+        sess = create_session()
+        try:
+            sess.save(d1)
+            assert False
+        except exceptions.AssertionError, err:
+            assert str(err) == "Attribute 'a' on class '%s' doesn't handle objects of type '%s'" % (D, B)
+
+class ViewOnlyTest(testbase.ORMTest):
+    """test a view_only mapping where a third table is pulled into the primary join condition,
+    using overlapping PK column names (should not produce "conflicting column" error)"""
+    def define_tables(self, metadata):
+        global t1, t2, t3
+        t1 = Table("t1", metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(40)))
+        t2 = Table("t2", metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(40)),
+            Column('t1id', Integer, ForeignKey('t1.id')))
+        t3 = Table("t3", metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(40)),
+            Column('t2id', Integer, ForeignKey('t2.id'))
+            )
+            
+    def test_basic(self):
+        class C1(object):pass
+        class C2(object):pass
+        class C3(object):pass
+        
+        mapper(C1, t1, properties={
+            't2s':relation(C2),
+            't2_view':relation(C2, viewonly=True, primaryjoin=and_(t1.c.id==t2.c.t1id, t3.c.t2id==t2.c.id, t3.c.data==t1.c.data))
+        })
+        mapper(C2, t2)
+        mapper(C3, t3, properties={
+            't2':relation(C2)
+        })
+        
+        c1 = C1()
+        c1.data = 'c1data'
+        c2a = C2()
+        c1.t2s.append(c2a)
+        c2b = C2()
+        c1.t2s.append(c2b)
+        c3 = C3()
+        c3.data='c1data'
+        c3.t2 = c2b
+        sess = create_session()
+        sess.save(c1)
+        sess.save(c3)
+        sess.flush()
+        sess.clear()
+        
+        c1 = sess.query(C1).get(c1.id)
+        assert set([x.id for x in c1.t2s]) == set([c2a.id, c2b.id])
+        assert set([x.id for x in c1.t2_view]) == set([c2b.id])
+
+class ViewOnlyTest2(testbase.ORMTest):
+    """test a view_only mapping where a third table is pulled into the primary join condition,
+    using non-overlapping PK column names (should not produce "mapper has no column X" error)"""
+    def define_tables(self, metadata):
+        global t1, t2, t3
+        t1 = Table("t1", metadata,
+            Column('t1id', Integer, primary_key=True),
+            Column('data', String(40)))
+        t2 = Table("t2", metadata,
+            Column('t2id', Integer, primary_key=True),
+            Column('data', String(40)),
+            Column('t1id_ref', Integer, ForeignKey('t1.t1id')))
+        t3 = Table("t3", metadata,
+            Column('t3id', Integer, primary_key=True),
+            Column('data', String(40)),
+            Column('t2id_ref', Integer, ForeignKey('t2.t2id'))
+            )
+    def test_basic(self):
+        class C1(object):pass
+        class C2(object):pass
+        class C3(object):pass
+
+        mapper(C1, t1, properties={
+            't2s':relation(C2),
+            't2_view':relation(C2, viewonly=True, primaryjoin=and_(t1.c.t1id==t2.c.t1id_ref, t3.c.t2id_ref==t2.c.t2id, t3.c.data==t1.c.data))
+        })
+        mapper(C2, t2)
+        mapper(C3, t3, properties={
+            't2':relation(C2)
+        })
+
+        c1 = C1()
+        c1.data = 'c1data'
+        c2a = C2()
+        c1.t2s.append(c2a)
+        c2b = C2()
+        c1.t2s.append(c2b)
+        c3 = C3()
+        c3.data='c1data'
+        c3.t2 = c2b
+        sess = create_session()
+        sess.save(c1)
+        sess.save(c3)
+        sess.flush()
+        sess.clear()
+
+        c1 = sess.query(C1).get(c1.t1id)
+        assert set([x.t2id for x in c1.t2s]) == set([c2a.t2id, c2b.t2id])
+        assert set([x.t2id for x in c1.t2_view]) == set([c2b.t2id])
+        
+        
 if __name__ == "__main__":
     testbase.main()        
